# -*- coding: utf-8 -*-

from subprocess import run
from io import BytesIO

import requests
from pandas import read_csv 

class HiddenMarkovModelLogo:
    """
    Contains logo information for a Hidden Markov Model as DataFrames. 
    These can be found as the heights, and indelinfo attributes. When 
    initializing the object it takes a PFAM accession ID.
    """
    def __init__(self, accession):
        self.accession = accession
        self.hmm = self.get_hmm(self.accession)
        self.heights, self.indelinfo = self.get_hmmlogo(self.hmm)

    def get_hmm(self, accession):
        """
        Return hidden Markov model from Pfam.
        """
        url = f"https://pfam.xfam.org/family/{accession}/hmm"
        response = requests.get(url=url)
        response.raise_for_status()
        hmm = response.content
        return hmm
    
    def get_hmmlogo(self, hmm):
        """
        Return hidden Markov model logo from hidden Markov model.
        """
        arguments = ["hmmlogo", "/dev/stdin"]
        process = run(arguments, input=hmm, capture_output=True)
        heights, indelinfo = self.read_hmmlogo(process.stdout)
        return heights, indelinfo
    
    def read_hmmlogo(self, hmmlogo_output):
        """
        Return heights and indelinfo from hidden Markov model logo.
        """
        column_names = ['profile']+list('ACDEFGHIKLMNPQRSTVWY-')
        seperator = ':?\s+\(?\s?'
        hmmlogo_df = read_csv(
            BytesIO(hmmlogo_output),
            index_col=0,
            skiprows=2,
            header=None,
            sep=seperator,
            names=column_names,
            engine="python",
        )
        hmmlogo_df = hmmlogo_df.drop('-', axis='columns')
        heights = self.clean_df(hmmlogo_df)
        indelinfo = self.get_indelinfo(hmmlogo_df)
        return heights, indelinfo
    
    def clean_df(self, df):
        """
        Remove nan columns and remaining rows containing nan values.
        """
        df = df.dropna(axis='columns', how='all')
        df = df.dropna(axis='rows', how='any')
        df = df.astype('float64')
        df.index = df.index.astype('int64')
        return df
    
    def get_indelinfo(self, hmmlogo_df):
        """
        Return indelinfo for hidden Markov model logo.
        """
        selection = hmmlogo_df['Y'].isna()
        indelinfo = hmmlogo_df[selection].copy()
        indelinfo = self.clean_df(indelinfo) 
        indelinfo = indelinfo.rename(
            columns=dict(
                A='insert probability', 
                C='average insert length', 
                D='occupancy',
            )
        )
        return indelinfo    



# -*- coding: utf-8 -*-

"""
hmmlogo.api
~~~~~~~~~~~

This module implements the hmmlogo API.

"""

from . import plot
from . import hmm

def get_svg(accession, **kwargs):
    logoplot = plot.LogoPlot(accession, **kwargs)
    svg = logoplot.get_svg()
    return svg

def save_svg(accession, filename, **kwargs):
    logoplot = plot.LogoPlot(accession, **kwargs)
    logoplot.fig.savefig(filename, format='svg')



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hmmlogo",
    version="0.0.4",
    author="Inti Manuel Yabar-Pagaza",
    author_email="intipagaza@live.dk",
    description="Package to plot logos for profile hidden Markov models.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/imyp/hmmlogo",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        'matplotlib',
        'requests',
        'pandas',
        'numpy',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)




# -*- coding: utf-8 -*-

import json
import io 
import base64
from os.path import dirname, abspath, join

import matplotlib.pyplot as plt
import numpy as np

from matplotlib.path import Path
from matplotlib.patches import PathPatch
from .hmm import HiddenMarkovModelLogo

# Paths for alphabet and colormap
here = dirname(abspath(__file__))
alph_path = join(here,'alph.json')
cmap_path = join(here,'cmap.json')

class LogoPlot(HiddenMarkovModelLogo):
    """
    Visualization of a profile hidden Markov model.
    """
    def __init__(self, accession, alph=alph_path, cmap=cmap_path):
        super().__init__(accession)
        with open(alph, 'r') as f:
            self.alph = json.load(f)
        with open(cmap, 'r') as f:
            self.cmap = json.load(f)
        self.fig, self.ax = self.initialize_plot()
        self.plot_heights(self.heights)
        self.plot_indelinfo(self.indelinfo)

    def initialize_plot(self):
        length = len(self.heights)
        fig, ax = plt.subplots(figsize=(length/2, 7))
        ax.set_xlim(0, length)
        return fig, ax
    
    def plot_heights(self, heights):
        y_max = 0
        for x_offset, (_, profile_heights) in enumerate(heights.iterrows()):
            self.plot_profile(profile_heights, x_offset)
            y_max = max(y_max, profile_heights.sum())
        self.ax.set_ylim(0, y_max)
    
    def plot_profile(self, profile, x_offset):
        y_offset = 0
        profile = profile.sort_values()
        for aminoacid, height in profile.iteritems():
            patch = self.get_aa_patch(
                aminoacid, 
                [1, height], 
                [x_offset, y_offset],
            )
            self.ax.add_patch(patch)
            y_offset += height
    
    def get_aa_patch(self, aminoacid, scale, offset):
        path = Path(*self.alph[aminoacid])
        path.vertices *= scale
        path.vertices += offset
        patch = PathPatch(path, color=self.cmap[aminoacid])
        return patch
    
    def plot_indelinfo(self, indelinfo):
        self.ax.xaxis.set_visible(False)
        indelinfo = indelinfo.round(decimals=2).T
        self.ax.table(
            cellText=indelinfo.values, 
            rowLabels=indelinfo.index, 
            colLabels=indelinfo.columns,
        )
        self.fig.tight_layout()

    def get_svg(self):
        image = io.StringIO()
        self.fig.savefig(image, format='svg', bbox_inches='tight')
        image.seek(0)
        image = "\n".join(image.readlines()[4:])
        return image

    def get_png(self):
        image = io.BytesIO()
        self.fig.savefig(image, format='png', bbox_inches='tight')
        image.seek(0)
        image = base64.b64encode(logofig.read()).decode()
        return image



# -*- coding: utf-8 -*-

"""
hmmlogo 
~~~~~~~

This package is used to visualize profile hidden Markov models.
Basic usage:

   >>> import hmmlogo
   >>> hmmlogo.get_svg('PF00010')
   <Figure Object>

"""

from .hmm import HiddenMarkovModelLogo
from .plot import LogoPlot

from .api import get_svg, save_svg
